@@ -28,7 +28,9 @@ module OneLogin
       attr_accessor :idp_cert
       attr_accessor :idp_cert_fingerprint
       attr_accessor :idp_cert_fingerprint_algorithm
+      attr_accessor :idp_cert_multi
       attr_accessor :idp_attribute_names
+      attr_accessor :idp_name_qualifier
       # SP Data
       attr_accessor :issuer
       attr_accessor :assertion_consumer_service_url
@@ -45,6 +47,7 @@ module OneLogin
       attr_accessor :attributes_index
       attr_accessor :force_authn
       attr_accessor :certificate
+      attr_accessor :certificate_new
       attr_accessor :private_key
       attr_accessor :authn_context
       attr_accessor :authn_context_comparison
@@ -93,7 +96,7 @@ module OneLogin
       end
 
       # Setter for Single Logout Service Binding.
-      # 
+      #
       # (Currently we only support "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect")
       # @param url [String]
       #
@@ -123,6 +126,32 @@ module OneLogin
         OpenSSL::X509::Certificate.new(formatted_cert)
       end
 
+      # @return [Hash with 2 arrays of OpenSSL::X509::Certificate] Build multiple IdP certificates from the settings.
+      #
+      def get_idp_cert_multi
+        return nil if idp_cert_multi.nil? || idp_cert_multi.empty?
+
+        raise ArgumentError.new("Invalid value for idp_cert_multi") if not idp_cert_multi.is_a?(Hash)
+
+        certs = {:signing => [], :encryption => [] }
+
+        if idp_cert_multi.key?(:signing) and not idp_cert_multi[:signing].empty?
+          idp_cert_multi[:signing].each do |idp_cert|
+            formatted_cert = OneLogin::RubySaml::Utils.format_cert(idp_cert)
+            certs[:signing].push(OpenSSL::X509::Certificate.new(formatted_cert))
+          end
+        end
+
+        if idp_cert_multi.key?(:encryption) and not idp_cert_multi[:encryption].empty?
+          idp_cert_multi[:encryption].each do |idp_cert|
+            formatted_cert = OneLogin::RubySaml::Utils.format_cert(idp_cert)
+            certs[:encryption].push(OpenSSL::X509::Certificate.new(formatted_cert))
+          end
+        end
+
+        certs
+      end
+
       # @return [OpenSSL::X509::Certificate|nil] Build the SP certificate from the settings (previously format it)
       #
       def get_sp_cert
@@ -132,6 +161,15 @@ module OneLogin
         OpenSSL::X509::Certificate.new(formatted_cert)
       end
 
+      # @return [OpenSSL::X509::Certificate|nil] Build the New SP certificate from the settings (previously format it)
+      #
+      def get_sp_cert_new
+        return nil if certificate_new.nil? || certificate_new.empty?
+
+        formatted_cert = OneLogin::RubySaml::Utils.format_cert(certificate_new)
+        OpenSSL::X509::Certificate.new(formatted_cert)
+      end
+
       # @return [OpenSSL::PKey::RSA] Build the SP private from the settings (previously format it)
       #
       def get_sp_key
@@ -151,15 +189,16 @@ module OneLogin
         :compress_response                         => true,
         :soft                                      => true,
         :security                                  => {
-          :authn_requests_signed    => false,
-          :logout_requests_signed   => false,
-          :logout_responses_signed  => false,
-          :want_assertions_signed   => false,
-          :want_name_id             => false,
-          :metadata_signed          => false,
-          :embed_sign               => false,
-          :digest_method            => XMLSecurity::Document::SHA1,
-          :signature_method         => XMLSecurity::Document::RSA_SHA1
+          :authn_requests_signed      => false,
+          :logout_requests_signed     => false,
+          :logout_responses_signed    => false,
+          :want_assertions_signed     => false,
+          :want_assertions_encrypted  => false,
+          :want_name_id               => false,
+          :metadata_signed            => false,
+          :embed_sign                 => false,
+          :digest_method              => XMLSecurity::Document::SHA1,
+          :signature_method           => XMLSecurity::Document::RSA_SHA1
         }.freeze,
         :double_quote_xml_attribute_values         => false,
       }.freeze
