@@ -32,7 +32,7 @@ class MetadataTest < Minitest::Test
       assert_equal "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress", REXML::XPath.first(xml_doc, "//md:NameIDFormat").text.strip
 
       assert_equal "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST", acs.attribute("Binding").value
-      assert_equal "https://foo.example/saml/consume", acs.attribute("Location").value      
+      assert_equal "https://foo.example/saml/consume", acs.attribute("Location").value
 
       assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
     end
@@ -75,7 +75,21 @@ class MetadataTest < Minitest::Test
       assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
     end
 
-    describe "when auth requests are signed" do
+    describe "WantAssertionsSigned" do
+      it "generates Service Provider Metadata with WantAssertionsSigned = false" do
+        settings.security[:want_assertions_signed] = false
+        assert_equal "false", spsso_descriptor.attribute("WantAssertionsSigned").value
+        assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
+      end
+
+      it "generates Service Provider Metadata with WantAssertionsSigned = true" do
+        settings.security[:want_assertions_signed] = true
+        assert_equal "true", spsso_descriptor.attribute("WantAssertionsSigned").value
+        assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
+      end
+    end
+
+    describe "with a sign/encrypt certificate" do
       let(:key_descriptors) do
         REXML::XPath.match(
           xml_doc,
@@ -97,35 +111,121 @@ class MetadataTest < Minitest::Test
         settings.certificate = ruby_saml_cert_text
       end
 
-      it "generates Service Provider Metadata with AuthnRequestsSigned" do
-        settings.security[:authn_requests_signed] = true
-        assert_equal "true", spsso_descriptor.attribute("AuthnRequestsSigned").value
+      it "generates Service Provider Metadata with X509Certificate for sign" do
+        assert_equal 1, key_descriptors.length
+        assert_equal "signing", key_descriptors[0].attribute("use").value
+
+        assert_equal 1, cert_nodes.length
         assert_equal ruby_saml_cert.to_der, cert.to_der
 
         assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
       end
 
-      it "generates Service Provider Metadata with X509Certificate for sign and encrypt" do
-        assert_equal 2, key_descriptors.length
-        assert_equal "signing", key_descriptors[0].attribute("use").value
-        assert_equal "encryption", key_descriptors[1].attribute("use").value
+      describe "and signed authentication requests" do
+        before do
+          settings.security[:authn_requests_signed] = true
+        end
 
-        assert_equal 2, cert_nodes.length
-        assert_equal ruby_saml_cert.to_der, cert.to_der
-        assert_equal cert_nodes[0].text, cert_nodes[1].text
+        it "generates Service Provider Metadata with AuthnRequestsSigned" do
+          assert_equal "true", spsso_descriptor.attribute("AuthnRequestsSigned").value
+          assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
+        end
+      end
 
-        assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
+      describe "and encrypted assertions" do
+        before do
+          settings.security[:want_assertions_encrypted] = true
+        end
+
+        it "generates Service Provider Metadata with X509Certificate for encrypt" do
+          assert_equal 2, key_descriptors.length
+
+          assert_equal "encryption", key_descriptors[1].attribute("use").value
+
+          assert_equal 2, cert_nodes.length
+          assert_equal cert_nodes[0].text, cert_nodes[1].text
+          assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
+        end
       end
     end
 
-    describe "when attribute service is configured" do
+    describe "with a future SP certificate" do
+      let(:key_descriptors) do
+        REXML::XPath.match(
+          xml_doc,
+          "//md:KeyDescriptor",
+          "md" => "urn:oasis:names:tc:SAML:2.0:metadata"
+        )
+      end
+      let(:cert_nodes) do
+        REXML::XPath.match(
+          xml_doc,
+          "//md:KeyDescriptor/ds:KeyInfo/ds:X509Data/ds:X509Certificate",
+          "md" => "urn:oasis:names:tc:SAML:2.0:metadata",
+          "ds" => "http://www.w3.org/2000/09/xmldsig#"
+        )
+      end
+
+      before do
+        settings.certificate = ruby_saml_cert_text
+        settings.certificate_new = ruby_saml_cert_text2
+      end
+
+      it "generates Service Provider Metadata with 2 X509Certificate for sign" do
+        assert_equal 2, key_descriptors.length
+        assert_equal "signing", key_descriptors[0].attribute("use").value
+        assert_equal "signing", key_descriptors[1].attribute("use").value
+
+        cert = OpenSSL::X509::Certificate.new(Base64.decode64(cert_nodes[0].text))
+        cert_new = OpenSSL::X509::Certificate.new(Base64.decode64(cert_nodes[1].text))
+
+        assert_equal 2, cert_nodes.length
+        assert_equal ruby_saml_cert.to_der, cert.to_der
+        assert_equal ruby_saml_cert2.to_der, cert_new.to_der
+
+        assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
+      end
+
+      describe "and signed authentication requests" do
+        before do
+          settings.security[:authn_requests_signed] = true
+        end
+
+        it "generates Service Provider Metadata with AuthnRequestsSigned" do
+          assert_equal "true", spsso_descriptor.attribute("AuthnRequestsSigned").value
+          assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
+        end
+      end
+
+      describe "and encrypted assertions" do
+        before do
+          settings.security[:want_assertions_encrypted] = true
+        end
+
+        it "generates Service Provider Metadata with X509Certificate for encrypt" do
+          assert_equal 4, key_descriptors.length
+          assert_equal "signing", key_descriptors[0].attribute("use").value
+          assert_equal "encryption", key_descriptors[1].attribute("use").value
+          assert_equal "signing", key_descriptors[2].attribute("use").value
+          assert_equal "encryption", key_descriptors[3].attribute("use").value
+
+          assert_equal 4, cert_nodes.length
+          assert_equal cert_nodes[0].text, cert_nodes[1].text
+          assert_equal cert_nodes[2].text, cert_nodes[3].text
+          assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
+        end
+      end
+
+    end
+
+    describe "when attribute service is configured with multiple attribute values" do
       let(:attr_svc)  { REXML::XPath.first(xml_doc, "//md:AttributeConsumingService") }
       let(:req_attr)  { REXML::XPath.first(xml_doc, "//md:RequestedAttribute") }
 
       before do
         settings.attribute_consuming_service.configure do
           service_name "Test Service"
-          add_attribute(:name => "Name", :name_format => "Name Format", :friendly_name => "Friendly Name", :attribute_value => "Attribute Value")
+          add_attribute(:name => 'Name', :name_format => 'Name Format', :friendly_name => 'Friendly Name', :attribute_value => ['Attribute Value One', false])
         end
       end
 
@@ -137,7 +237,35 @@ class MetadataTest < Minitest::Test
         assert_equal "Name", req_attr.attribute("Name").value
         assert_equal "Name Format", req_attr.attribute("NameFormat").value
         assert_equal "Friendly Name", req_attr.attribute("FriendlyName").value
-        assert_equal "Attribute Value", REXML::XPath.first(xml_doc, "//saml:AttributeValue").text.strip
+
+        attribute_values = REXML::XPath.match(xml_doc, "//saml:AttributeValue").map(&:text)
+        assert_equal "Attribute Value One", attribute_values[0]
+        assert_equal 'false', attribute_values[1]
+
+        assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
+      end
+    end
+
+    describe "when attribute service is configured" do
+      let(:attr_svc) { REXML::XPath.first(xml_doc, '//md:AttributeConsumingService') }
+      let(:req_attr) { REXML::XPath.first(xml_doc, '//md:RequestedAttribute') }
+
+      before do
+        settings.attribute_consuming_service.configure do
+          service_name "Test Service"
+          add_attribute(:name => 'active', :name_format => 'format', :friendly_name => 'Active', :attribute_value => true)
+        end
+      end
+
+      it "generates attribute service" do
+        assert_equal "true", attr_svc.attribute("isDefault").value
+        assert_equal "1", attr_svc.attribute("index").value
+        assert_equal REXML::XPath.first(xml_doc, "//md:ServiceName").text.strip, "Test Service"
+
+        assert_equal 'active', req_attr.attribute('Name').value
+        assert_equal 'format', req_attr.attribute('NameFormat').value
+        assert_equal 'Active', req_attr.attribute('FriendlyName').value
+        assert_equal 'true', REXML::XPath.first(xml_doc, '//saml:AttributeValue').text.strip
 
         assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
       end
@@ -175,7 +303,7 @@ class MetadataTest < Minitest::Test
         assert_match %r[<ds:SignatureMethod Algorithm='http://www.w3.org/2000/09/xmldsig#rsa-sha1'/>], xml_text
         assert_match %r[<ds:DigestMethod Algorithm='http://www.w3.org/2000/09/xmldsig#sha1'/>], xml_text
         signed_metadata = XMLSecurity::SignedDocument.new(xml_text)
-        assert signed_metadata.validate_document(ruby_saml_cert_fingerprint, false)        
+        assert signed_metadata.validate_document(ruby_saml_cert_fingerprint, false)
 
         assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
       end
@@ -189,11 +317,11 @@ class MetadataTest < Minitest::Test
         it "creates a signed metadata with specified digest and signature methods" do
           assert_match %r[<ds:SignatureValue>([a-zA-Z0-9/+=]+)</ds:SignatureValue>]m, xml_text
           assert_match %r[<ds:SignatureMethod Algorithm='http://www.w3.org/2001/04/xmldsig-more#rsa-sha256'/>], xml_text
-          assert_match %r[<ds:DigestMethod Algorithm='http://www.w3.org/2001/04/xmldsig-more#sha512'/>], xml_text
+          assert_match %r[<ds:DigestMethod Algorithm='http://www.w3.org/2001/04/xmlenc#sha512'/>], xml_text
 
           signed_metadata_2 = XMLSecurity::SignedDocument.new(xml_text)
 
-          assert signed_metadata_2.validate_document(ruby_saml_cert_fingerprint, false)          
+          assert signed_metadata_2.validate_document(ruby_saml_cert_fingerprint, false)
 
           assert validate_xml!(xml_text, "saml-schema-metadata-2.0.xsd")
         end
