@@ -2,6 +2,7 @@ require 'simplecov'
 
 SimpleCov.start do
   add_filter "test/"
+  add_filter "vendor/"
   add_filter "lib/onelogin/ruby-saml/logging.rb"
 end
 
@@ -128,7 +129,15 @@ class Minitest::Test
   end
 
   def unsigned_message_encrypted_unsigned_assertion
-    @unsigned_message_encrypted_unsigned_assertion ||= File.read(File.join(File.dirname(__FILE__), 'responses', 'unsigned_message_encrypted_unsigned_assertion.xml.base64'))    
+    @unsigned_message_encrypted_unsigned_assertion ||= File.read(File.join(File.dirname(__FILE__), 'responses', 'unsigned_message_encrypted_unsigned_assertion.xml.base64'))
+  end
+
+  def response_document_encrypted_attrs
+    @response_document_encrypted_attrs ||= File.read(File.join(File.dirname(__FILE__), 'responses', 'response_encrypted_attrs.xml.base64'))
+  end
+
+  def response_document_double_status_code
+    @response_document_double_status_code ||= File.read(File.join(File.dirname(__FILE__), 'responses', 'response_double_status_code.xml.base64'))
   end
 
   def signature_fingerprint_1
@@ -145,8 +154,40 @@ class Minitest::Test
     @certificate_without_head_foot ||= read_certificate("certificate_without_head_foot")
   end
 
-  def idp_metadata
-    @idp_metadata ||= read_response("idp_descriptor.xml")
+  def idp_metadata_descriptor
+    @idp_metadata_descriptor ||= File.read(File.join(File.dirname(__FILE__), 'metadata', 'idp_descriptor.xml'))
+  end
+
+  def idp_metadata_descriptor2
+    @idp_metadata_descriptor2 ||= File.read(File.join(File.dirname(__FILE__), 'metadata', 'idp_descriptor_2.xml'))
+  end
+
+  def idp_metadata_descriptor3
+    @idp_metadata_descriptor3 ||= File.read(File.join(File.dirname(__FILE__), 'metadata', 'idp_descriptor_3.xml'))
+  end
+
+  def no_idp_metadata_descriptor
+    @no_idp_metadata_descriptor ||= File.read(File.join(File.dirname(__FILE__), 'metadata', 'no_idp_descriptor.xml'))
+  end
+
+  def idp_metadata_multiple_descriptors
+    @idp_metadata_multiple_descriptors ||= File.read(File.join(File.dirname(__FILE__), 'metadata', 'idp_multiple_descriptors.xml'))
+  end
+
+  def idp_metadata_multiple_certs
+    @idp_metadata_multiple_descriptors ||= File.read(File.join(File.dirname(__FILE__), 'metadata', 'idp_metadata_multi_certs.xml'))
+  end
+
+  def idp_metadata_multiple_signing_certs
+    @idp_metadata_multiple_signing_certs ||= File.read(File.join(File.dirname(__FILE__), 'metadata', 'idp_metadata_multi_signing_certs.xml'))
+  end
+
+  def idp_metadata_same_sign_and_encrypt_cert
+    @idp_metadata_same_sign_and_encrypt_cert ||= File.read(File.join(File.dirname(__FILE__), 'metadata', 'idp_metadata_same_sign_and_encrypt_cert.xml'))
+  end
+
+  def idp_metadata_different_sign_and_encrypt_cert
+    @idp_metadata_different_sign_and_encrypt_cert ||= File.read(File.join(File.dirname(__FILE__), 'metadata', 'idp_metadata_different_sign_and_encrypt_cert.xml'))
   end
 
   def logout_request_document
@@ -158,6 +199,15 @@ class Minitest::Test
     @logout_request_document
   end
 
+  def logout_request_document_with_name_id_format
+    unless @logout_request_document_with_name_id_format
+      xml = read_logout_request("slo_request_with_name_id_format.xml")
+      deflated = Zlib::Deflate.deflate(xml, 9)[2..-5]
+      @logout_request_document_with_name_id_format = Base64.encode64(deflated)
+    end
+    @logout_request_document_with_name_id_format
+  end
+
   def logout_request_xml_with_session_index
     @logout_request_xml_with_session_index ||= File.read(File.join(File.dirname(__FILE__), 'logout_requests', 'slo_request_with_session_index.xml'))
   end
@@ -183,6 +233,10 @@ class Minitest::Test
     @ruby_saml_cert ||= OpenSSL::X509::Certificate.new(ruby_saml_cert_text)
   end
 
+  def ruby_saml_cert2
+    @ruby_saml_cert2 ||= OpenSSL::X509::Certificate.new(ruby_saml_cert_text2)
+  end
+
   def ruby_saml_cert_fingerprint
     @ruby_saml_cert_fingerprint ||= Digest::SHA1.hexdigest(ruby_saml_cert.to_der).scan(/../).join(":")
   end
@@ -191,6 +245,10 @@ class Minitest::Test
     read_certificate("ruby-saml.crt")
   end
 
+  def ruby_saml_cert_text2
+    read_certificate("ruby-saml-2.crt")
+  end
+
   def ruby_saml_key
     @ruby_saml_key ||= OpenSSL::PKey::RSA.new(ruby_saml_key_text)
   end
